#pragma once

#include <cinttypes>
#include <cstdint>
#include <cstring>
#include <exception>
#include <string>

#include <args.hxx>

#include "common.hh"


namespace blky::args {

using namespace ::args;


struct uvec2_Reader final {
  void operator()(const std::string&, const std::string& value, uvec2& dst) {
    char _;
    if (3 != sscanf(value.c_str(), "%" PRIu32 "%c%" PRIu32, &dst.x, &_, &dst.y)) {
      throw std::runtime_error {"invalid uvec2 specification: "+value};
    }
  }
};


/* ---- root parser ---- */
static inline ArgumentParser parser {
  "blocky is a command line tool for data hiding of video"
};
static inline Group commands { parser, "COMMAND:" };
static inline HelpFlag help { parser, "help", "display this help menu", {'h', "help"}, Options::Global };


/* ---- group: key ---- */
static inline Group key { "key information:" };
static inline ValueFlag<uvec2, uvec2_Reader> block_num {
  key, "8x8", "number of blocks", {"block-num"}, {8, 8}
};
static inline ValueFlag<uint32_t> feat_bits {
  key, "1", "bits per each step", {"feat-bits"}, 1
};
static inline ValueFlag<uint32_t> seed {
  key, "123", "random seed for hopping", {"seed"}, 123
};
static inline ValueFlag<uint32_t> utime {
  key, "10", "duration of each step (frames)", {"utime"}, 10
};


/* ---- group: stream source ---- */
static inline Group stream { "stream source (choose one):", Group::Validators::AtMostOne };
static inline Flag stream_stdin {
  stream, "stdin", "load from stdin (default)", {"stdin"}, true
};
static inline Flag stream_stdin_hex {
  stream, "stdin-hex", "load from stdin as HEX (e.g 'FF00'", {"stdin-hex"}
};


/* ---- command: embed ---- */
static inline Group embed {};
static inline Positional<std::string> embed_dst {
  embed, "dst", "destination video path", Options::Required
};
static inline Positional<std::string> embed_src {
  embed, "src", "source video path", Options::Required
};
static inline Command command_embed { commands, "embed", "embed stream to video", [](auto& p) {
  p.Add(embed);
  p.Add(stream);
  p.Add(key);
  p.Parse();
  Cmd_Embed();
}};


/* ---- command: extract ---- */
static inline Group extract {};
static inline Positional<std::string> extract_src {
  extract, "src", "source video path", Options::Required
};
static inline Command command_extract { commands, "extract", "extract stream from video", [](auto& p) {
  p.Add(extract);
  p.Add(key);
  p.Parse();
  Cmd_Extract();
}};

}  // namespace blky::args
